﻿using System;
using EFCoreAddRelationship.Logger;
using Microsoft.EntityFrameworkCore;
using Microsoft.EntityFrameworkCore.Metadata;

namespace EFCoreAddRelationship.Entities
{
    public partial class TestDBContext : DbContext
    {
        public TestDBContext()
        {
        }

        public TestDBContext(DbContextOptions<TestDBContext> options)
            : base(options)
        {
        }

        public virtual DbSet<Book> Book { get; set; }
        public virtual DbSet<Person> Person { get; set; }

        protected override void OnConfiguring(DbContextOptionsBuilder optionsBuilder)
        {
            if (!optionsBuilder.IsConfigured)
            {
                optionsBuilder.UseLoggerFactory(new EFLoggerFactory());
                optionsBuilder.UseSqlServer("Server=localhost;User Id=sa;Password=Dtt!123456;Database=TestDB");
            }
        }

        protected override void OnModelCreating(ModelBuilder modelBuilder)
        {
            modelBuilder.Entity<Book>(entity =>
            {
                entity.HasKey(e => e.BookCode);//设置Book实体的BookCode属性为EF Core实体的Key属性

                entity.HasIndex(e => e.BookCode)
                    .HasName("IX_Book")
                    .IsUnique();

                entity.Property(e => e.Id).ValueGeneratedOnAdd();//设置Book实体的Id属性为插入数据到数据库Book表时自动生成，因为Book表的ID列为自增列
                entity.Property(e => e.Id).HasColumnName("ID");

                entity.Property(e => e.BookCode).HasMaxLength(20);

                entity.Property(e => e.BookName).HasMaxLength(50);

                entity.Property(e => e.Isbn)
                    .HasColumnName("ISBN")
                    .HasMaxLength(20);

                entity.Property(e => e.PersonCode).HasMaxLength(20);
            });

            modelBuilder.Entity<Person>(entity =>
            {
                entity.HasKey(e => e.PersonCode);//设置Person实体的PersonCode属性为EF Core实体的Key属性

                entity.HasIndex(e => e.PersonCode)
                    .HasName("IX_Person")
                    .IsUnique();

                entity.Property(e => e.Id).ValueGeneratedOnAdd();//设置Person实体的Id属性为插入数据到数据库Person表时自动生成，因为Person表的ID列为自增列
                entity.Property(e => e.Id).HasColumnName("ID");

                entity.Property(e => e.Name).HasMaxLength(50);

                entity.Property(e => e.PersonCode).HasMaxLength(20);

                //设置Person实体和Book实体之间的一对多关系，尽管我们并没有在数据库中建立Person表和Book表之间的一对多外键关系，但是我们可以用EF Core的Fluent API在实体层面设置外键关系
                entity.HasMany(p => p.Book)//设置Person实体通过属性Book可以找到多个Book实体，表示Person表是一对多关系中的主表
                .WithOne(b => b.Person)//设置Book实体通过属性Person可以找到一个Person实体，表示Book表是一对多关系中的从表
                .HasPrincipalKey(p => p.PersonCode)//设置Person表的PersonCode列为一对多关系中的主表键
                .HasForeignKey(b => b.PersonCode)//设置Book表的PersonCode列为一对多关系中的从表外键
                .OnDelete(DeleteBehavior.ClientSetNull);//设置一对多关系的级联删除效果为DeleteBehavior.ClientSetNull

            });
        }
    }
}
